@@ -43,6 +43,8 @@ Components.utils.import("resource://requestpolicy/Util.jsm",
  */
 requestpolicy.overlay = {
 
+  supportRequestPolicyURL : "https://www.requestpolicy.com/support-requestpolicy.html",
+
   _extensionConflictInfoUri : "http://www.requestpolicy.com/conflict?ext=",
 
   _prefetchInfoUri : "http://www.requestpolicy.com/help/prefetch.html",
@@ -1291,7 +1293,7 @@ requestpolicy.overlay = {
     if (event.currentTarget != event.originalTarget) {
       return;
     }
-    // Leave the popup attached to the context menu, as we consdier that the
+    // Leave the popup attached to the context menu, as we consider that the
     // default location for it.
     this._attachPopupToContextMenu();
   },
@@ -1550,6 +1552,10 @@ requestpolicy.overlay = {
         + encodeURIComponent(extJson));
   },
 
+  openSupportRP : function() {
+    this._openInNewTab(this.supportRequestPolicyURL);
+  },
+
   showPrefetchInfo : function() {
     this._openInNewTab(this._prefetchInfoUri);
   },
@@ -1626,8 +1632,7 @@ requestpolicy.overlay = {
       openRequestLog.hidden = false;
       this.requestLogTreeView = null;
     }
-  }
-
+  },
 };
 
 // Initialize the requestpolicy.overlay object when the window DOM is loaded.
