@@ -357,6 +357,8 @@ RequestPolicyService.prototype = {
         "https://www.google.com/recaptcha/api/challenge?", appInfo.vendor]);
     this._compatibilityRules.push(["https://auth.services.mozilla.com/",
         "https://www.google.com/recaptcha/api/", appInfo.vendor]);
+    // Firefox 13 added links from about:newtab
+    this._compatibilityRules.push(["about:newtab", null, appInfo.vendor]);
 
     // Flock
     if (appInfo.ID == "{a463f10c-3994-11da-9945-000d60ca027b}") {
@@ -380,6 +382,12 @@ RequestPolicyService.prototype = {
     }
   },
 
+  _clearPref: function(name) {
+    if (this.prefs.prefHasUserValue(name)) {
+      this.prefs.clearUserPref(name);
+    }
+  },
+
   _syncFromPrefs : function() {
     // Load the logging preferences before the others.
     this._updateLoggingSettings();
@@ -428,16 +436,30 @@ RequestPolicyService.prototype = {
       }
     }
 
-    // Clean up old, unused prefs (removed in 0.2.0).
-    deletePrefs = ["temporarilyAllowedOrigins",
-        "temporarilyAllowedDestinations",
-        "temporarilyAllowedOriginsToDestinations"];
-    for (var i = 0; i < deletePrefs.length; i++) {
-      if (this.prefs.prefHasUserValue(deletePrefs[i])) {
-        this.prefs.clearUserPref(deletePrefs[i]);
-      }
+    // Study is over.
+    if (this.prefs.getBoolPref('study.participate')) {
+      this.endParticipationInStudy();
     }
+    this._clearPref('study.participate');
+    this._clearPref('study.profileID');
+    this._clearPref('study.consentID');
+    this._clearPref('study.consentVersion');
+    this._clearPref('study.sessionID');
+    this._clearPref('study.globalEventID');
     this._prefService.savePrefFile(null);
+
+    // Clean up old, unused prefs (removed in 0.2.0).
+    this._clearPref("temporarilyAllowedOrigins");
+    this._clearPref("temporarilyAllowedDestinations");
+    this._clearPref("temporarilyAllowedOriginsToDestinations");
+    this._prefService.savePrefFile(null);
+  },
+
+  endParticipationInStudy : function() {
+    this.prefs.setBoolPref('study.participate', false);
+    this._clearPref('study.consentVersion');
+    this._prefService.savePrefFile(null);
+    requestpolicy.mod.Stats.deleteFile();
   },
 
   _updateLoggingSettings : function() {
@@ -613,6 +635,8 @@ RequestPolicyService.prototype = {
         requestpolicy.mod);
     Components.utils.import("resource://requestpolicy/Util.jsm",
         requestpolicy.mod);
+    Components.utils.import("resource://requestpolicy/Stats.jsm",
+        requestpolicy.mod);
     try {
       Components.utils.import("resource://gre/modules/AddonManager.jsm");
     } catch (e) {
@@ -1297,6 +1321,9 @@ RequestPolicyService.prototype = {
   },
 
   _forbidOriginToDestinationByCombinedIdentifier : function(combinedId, noStore) {
+    var parts = combinedId.split('|');
+    var originIdentifier = parts[0];
+    var destIdentifier = parts[1];
     if (this._temporarilyAllowedOriginsToDestinations[combinedId]) {
       this._temporarilyAllowedOriginsToDestinationsCount--;
       delete this._temporarilyAllowedOriginsToDestinations[combinedId];
@@ -1641,6 +1668,7 @@ RequestPolicyService.prototype = {
         // what is needed to allow their requests.
         this._initializeExtensionCompatibility();
         this._initializeApplicationCompatibility();
+
         break;
       case "private-browsing" :
         if (data == "enter") {
@@ -2001,7 +2029,7 @@ RequestPolicyService.prototype = {
 
     // https://developer.mozilla.org/en/nsIContentPolicy
     shouldLoad : function(aContentType, aContentLocation, aRequestOrigin,
-        aContext, aMimeTypeGuess, aExtra) {
+        aContext, aMimeTypeGuess, aExtra, aRequestPrincipal) {
       try {
 
         if (this._isInternalRequest(aContentLocation, aRequestOrigin)) {
@@ -2017,6 +2045,25 @@ RequestPolicyService.prototype = {
         var dest = requestpolicy.mod.DomainUtil
             .stripFragment(aContentLocation.spec);
 
+        // Fx 16 changed the following: 1) we should be able to count on the
+        // referrer (aRequestOrigin) being set to something besides
+        // moz-nullprincipal when there is a referrer, and 2) the new argument
+        // aRequestPrincipal is provided. This means our hackery to set the
+        // referrer based on aContext when aRequestOrigin is moz-nullprincipal
+        // is now causing requests that don't have a referrer (namely, URLs
+        // entered in the address bar) to be blocked and trigger a top-level
+        // document redirect notification.
+        if (aRequestOrigin.scheme == "moz-nullprincipal" && aRequestPrincipal) {
+          requestpolicy.mod.Logger.warning(
+              requestpolicy.mod.Logger.TYPE_CONTENT,
+              "Allowing request that appears to be a URL entered in the "
+                  + "location bar or some other good explanation: " + dest);
+          return CP_OK;
+        }
+
+        // Note: Assuming the Fx 16 moz-nullprincipal+aRequestPrincipal check
+        // above is correct, this should be able to be removed when Fx < 16 is
+        // no longer supported.
         if (aRequestOrigin.scheme == "moz-nullprincipal" && aContext) {
           var newOrigin = requestpolicy.mod.DomainUtil
                 .stripFragment(aContext.contentDocument.documentURI);
@@ -2028,6 +2075,16 @@ RequestPolicyService.prototype = {
           aRequestOrigin = requestpolicy.mod.DomainUtil.getUriObject(origin);
         }
 
+        if (aRequestOrigin.scheme == "view-source") {
+          var newOrigin = origin.split(":").slice(1).join(":");
+          requestpolicy.mod.Logger.info(
+            requestpolicy.mod.Logger.TYPE_CONTENT,
+            "Considering view-source origin <"
+              + origin + "> to be origin <" + newOrigin + ">");
+          origin = newOrigin;
+          aRequestOrigin = requestpolicy.mod.DomainUtil.getUriObject(origin);
+        }
+
         if (aContentLocation.scheme == "view-source") {
           var newDest = dest.split(":").slice(1).join(":");
           requestpolicy.mod.Logger.info(
